/**
 *   @file     AWError.h
 *   @date     2020-11-09
 *   @author   whiothes <whiothes81@gmail.com>
 *   @version  1.0
 *   @brief    error definitions
 */
#pragma once

#include <errno.h>

/**
 *  @brief      error no. - error description MAP
 *
 *  @note       add your own (error macro, error description) map here
 */
#define AW_ERRNO_MAP(XX)                              \
  /* no error */                                      \
  XX(SUCC, "SUCCESS")                                 \
                                                      \
  /* use system errno */                              \
  XX(ERR_SYS, "Use strerror instead")                 \
                                                      \
  /* SQLite error */                                  \
  XX(ERR_SQLITE, "Call sqlite_errmsg() after")        \
  XX(ERR_SQLITE_NOT_FOUND, "No result found")         \
                                                      \
  /* MySQL error */                                   \
  XX(ERR_MYSQL, "Call mysql_error() for detail ")     \
                                                      \
  /* Custom error */                                  \
  XX(ERR_INVALID_ADDR, "Invalid memory address")      \
  XX(ERR_INVALID_TYPE, "Invalid data type")           \
  XX(ERR_NO_MEMORY, "No enough memory")               \
  XX(ERR_NO_FILE, "Check if file exists")             \
  XX(ERR_NO_DATA, "No data found")                    \
                                                      \
  XX(ERR_SQL_INVALID_METHOD, "Invalid sql method")    \
  XX(ERR_SQL_CONN, "Connect error, check parameters") \
                                                      \
  XX(ERR_INVALID_LENGTH, "Invalid length")            \
  XX(ERR_INVALID_DATA, "Invalid data")                \
  XX(ERR_INVALID_INDEX, "Invalid index")              \
  /* unknown error */                                 \
  XX(ERR_UNKNOWN, "Unexpected error occurred")

/**
 *  @brief      Define AW_* values for each errno value above
 *
 *  @warning    DO NOT DO ANY EDIT HERE
 */
#define AW_ERRNO_GEN(n, s) AW_##n,

/**
 *  @brief      make ERROR macro to be enum
 *
 *  @warning    DO NOT DO ANY EDIT HERE
 */
typedef enum { AW_FAIL = -1, AW_ERRNO_MAP(AW_ERRNO_GEN) } AW_errno;
#undef AW_ERRNO_GEN

const char *AW_ERR_desc(AW_errno errnum) __attribute__((deprecated("use AW_strerror instead")));
const char *AW_strerror(AW_errno errnum);

extern _Thread_local AW_errno AW_errnum;
